from flask import Blueprint, request, jsonify, make_response, json
from app.customer.schemas import customer_schema
from app.customer.models import Customer
from app.utils import token_required, delete_record, paginate_query
from app.customer.utills import process_image, serialize_customer, handle_db_commit
from app.devices.models import Device
from app.gateways.models import Gateways
from app.logs.models import Alert_logs, AlertLogsHistory, AlertHistory
from app.patients.models import Patient
from app.rooms.models import Room
from app.Site.models import Site
from app.users.models import User
from app.location_log.models import LocationLogHistory
from app.zone.models import Zone

customer = Blueprint('customer', __name__)
@customer.route('/api/v1/add/customer', methods=['POST'])
@token_required(1)
def create_customer(current_user):
    try:
        customer_dict = json.loads(request.form.get('formData'))
        image_data, logo_content_type = process_image(request.files)
        return handle_db_commit(Customer(**customer_schema.load({
            'customer_name': customer_dict.get('customer_name', '').strip(),
            'email': customer_dict.get('email', '').strip(),
            'description': customer_dict.get('description', '').strip(),
            'address': customer_dict.get('address', '').strip(),
            'logo': image_data,
            'logo_content_type': logo_content_type,
            'status': customer_dict.get('status', '').strip()
        })), 'create')
    except Exception as e:
        return make_response(jsonify({'message': 'Failed to add customer', "status": "error", 'error': str(e)}), 500)
@customer.route('/customer', methods=['GET'])
@token_required(1)
def get_customer_detail(current_user):
    try:
        search = request.args.get('search')
        customers = Customer.query.filter(Customer.customer_name.ilike(f'%{search}%')).all() if search else Customer.query.filter_by(status='active').all()
        serialized_customers = [serialize_customer(cust) for cust in customers if cust.customer_id != 1]
        if serialized_customers:
            return jsonify({"customers": serialized_customers}), 200
        else:
            return jsonify({"error": "No Customers Found!"}), 200
    except Exception as e:
        return make_response(jsonify({"Error": str(e)}), 500)

@customer.route('/api/v1/get/customer', methods=['GET'])
@token_required(1)
def get_customer(current_user):
    try:
        page = request.args.get('page', 1, type=int)
        per_page = request.args.get('per_page', 10, type=int)
        search = request.args.get('search', '')
        customers_query = Customer.query.filter(Customer.customer_name.ilike(f'%{search}%')).order_by(
            Customer.customer_name.asc()) if search else Customer.query.order_by(Customer.customer_name.asc())
        customers, pagination_data = paginate_query(customers_query, page, per_page)
        serialized_customers = [serialize_customer(cust) for cust in customers if cust.customer_id != 1]
        if serialized_customers:
            return make_response(jsonify({'customer': serialized_customers, "data": pagination_data}), 200)
        else:
            return make_response(jsonify({'message': 'No customers found.', "data": pagination_data, "status": "success"}), 200)
    except Exception as e:
        return make_response(jsonify({'message': f'Error: {e}', "status": "error"}), 200)

@customer.route('/api/v1/get/customer_by_id/<int:customer_id>', methods=['GET'])
@token_required(1)
def get_customer_by_id(current_user, customer_id):
    try:
        customer = Customer.query.filter_by(customer_id=customer_id).first()
        if customer:
            serialized_customer = serialize_customer(customer)
            logo = {
                'logo': serialized_customer['logo'],
                'logo_content_type': serialized_customer['logo_content_type'],
            }
            serialized_customer.pop('logo_content_type')
            serialized_customer.pop('logo')
            return make_response(jsonify({'customer': serialized_customer, "logo": logo}), 200)
        return make_response(jsonify({'message': 'customer not found', "status": "error"}), 500)
    except Exception as e:
        return make_response(jsonify({'message': f'error: {e}', "status": "error"}), 500)
@customer.route('/api/v1/update/customer/<int:customer_id>', methods=['PUT'])
@token_required(1)
def update_customer(current_user, customer_id):
    try:
        customer = Customer.query.get(customer_id)
        if not customer:
            return make_response(jsonify({'message': 'Customer not found', "status": "error"}), 404)
        customer_dict = json.loads(request.form.get('formData'))
        image_data, logo_content_type = process_image(request.files)
        if image_data:
            customer.logo = image_data
            customer.logo_content_type = logo_content_type
        for key, value in customer_schema.load({
            'customer_name': customer_dict.get('customer_name').strip(),
            'email': customer_dict.get('email').strip(),
            'description': customer_dict.get('description').strip(),
            'address': customer_dict.get('address').strip(),
            'status': customer_dict.get('status').strip(),
        }).items():
            setattr(customer, key, value)
        return handle_db_commit(customer, 'update')
    except Exception as e:
        return make_response(jsonify({'message': 'Failed to update customer', "status": "error", 'error': str(e)}), 500)
@customer.route('/api/v1/delete/customer/<int:customer_id>', methods=['DELETE'])
@token_required(1)
def delete_customer(current_user, customer_id):
    sites= Site.query.filter_by(customer_id=customer_id).all()
    for site in sites:
        Device.query.filter_by(site_id=site.site_id).delete()
        Gateways.query.filter_by(site_id=site.site_id).delete()
        Alert_logs.query.filter_by(site_id=site.site_id).delete()
        Patient.query.filter_by(site_id=site.site_id).delete()
        Room.query.filter_by(site_id=site.site_id).delete()
        Site.query.filter_by(site_id=site.site_id).delete()
        User.query.filter_by(site_id=site.site_id).delete()
        AlertLogsHistory.query.filter_by(site_id=site.site_id).delete()
        AlertHistory.query.filter_by(site_id=site.site_id).delete()
        LocationLogHistory.query.filter_by(site_id=site.site_id).delete()
        Zone.query.filter_by(site_id=site.site_id).delete()
    return delete_record(Customer, current_user, customer_id, "Organisation", "customer_id")